#!/usr/bin/env python
# -*-coding:utf-8-*-


from . import __softname__

DEFAULT_DICT = 'dict_small.txt'

DEFAULT_HMM_DATA = 'hmm_data.json'

DEFALUT_CACHE_NAME = f'{__softname__}.cache'

CACHE_WRITING = {}